/**
 * COPYRIGHT 2019 Flirc, Inc. All rights reserved.
 *
 * This copyright notice is Copyright Management Information under 17 USC 1202
 * and is included to protect this work and deter copyright infringement.
 * Removal or alteration of this Copyright Management Information without
 * the express written permission of Flirc, Inc. is prohibited, and any
 * such unauthorized removal or alteration will be a violation of federal law.
 *
 * @file
 * @brief   Transport libraries
 */

#ifndef I__TRANSPORT_H__
#define I__TRANSPORT_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <prjutil.h>
#include <stdint.h>

/* Total Packet length, we pack command packets in this */
#define PKV_DATA_LEN			128
/* Limit a command to 52 bytes of data. 64 bytes - str+status */
#define CMD_DATA_LEN			52
/* String length. 10+1 (Null Character) */
#define CMD_LENGTH			11

struct dev_t {
	int (*write)(struct dev_t *, uint8_t *buf, uint32_t len);
	int (*read)(struct dev_t *,  uint8_t *buf, uint32_t len);
	int (*send_pkv)(struct dev_t *, uint8_t type, uint8_t *buf, uint32_t len);
	int (*recv_pkv)(struct dev_t *, uint8_t type, uint8_t *buf, uint32_t len);
	int (*flush)(struct dev_t *);
	int (*close)(struct dev_t *);
	int (*send_cmd)(struct dev_t *, const char *, uint8_t *, uint32_t);
	void *handle;
	void *priv;
};

struct __attribute__((packed)) pkt_t {
	uint8_t type;
	uint8_t len;
	uint8_t data[PKV_DATA_LEN];
} __packed __alias;

struct cmd_pkt_t {
	char cmd[CMD_LENGTH];
	uint8_t status;
	uint8_t data[CMD_DATA_LEN];
} __packed __alias;

#define PKT_H_SIZE() \
	(sizeof(((struct cmd_pkt_t *)NULL)->cmd) + \
	sizeof(((struct cmd_pkt_t *)NULL)->status))

typedef enum {
	PKTYPE_NONE = 0,
	PKTYPE_RESET,
	PKTYPE_CMD,
	PKTYPE_DATA,
	PKTYPE_INVALID = 0xFF
} PKTYPE_ENUM;

/**
 * dev_send_pkv() send a command packet
 *
 * @param dev   - pointer to device
 * @param type 	- packet type
 * @param *buf  - pointer to buffer to write
 * @param len 	- length of buffer
 *
 * @return      int	- 0 Success, < 0 Error
 */
int dev_send_pkv(struct dev_t *dev, uint8_t type, uint8_t *buf, uint32_t len);

/**
 * dev_read_pkv() read packet
 *
 * @param dev   - pointer to device
 * @param type 	- packet type
 * @param *buf  - pointer to buffer to write
 * @param len 	- length of buffer
 *
 * @return      int	- 0 Success, < 0 Error
 */
int dev_revc_pkv(struct dev_t *dev, uint8_t type, uint8_t *buf, uint32_t len);

/**
 * dev_write() write to device
 *
 * @param dev   - pointer to device
 * @param *buf  - pointer to buffer to write
 * @param len 	- length of buffer
 *
 * @return      int	- 0 Success, < 0 Error
 */
int dev_write(struct dev_t *dev, uint8_t *buf, uint32_t len);

/**
 * dev_read() reads from device
 *
 * @param dev   - pointer to device
 * @param *buf  - pointer to buffer to write
 * @param len 	- length of buffer
 *
 * @return      int	- 0 Success, < 0 Error
 */
int dev_read(struct dev_t *dev, uint8_t *buf, uint32_t len);

/**
 * dev_flush() flush the device
 *
 * @param dev   - pointer to device
 *
 * @return      None.
 */
int dev_flush(struct dev_t *dev);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* I__TRANSPORT_H__ */
